numbers = """
zero
one
two
three
four
five
six
seven
eight
nine
ten
eleven
twelve
dozen
thirteen
fourteen
fifteen
sixteen
seventeen
eighteen
nineteen
twenty
thirty
fourty
fifty
sixty
seventy
eighty
ninety
hundred
thousand
million
billion
trillion
"""
