X = """
anus - butt
arse - butt
arsehole - butt
ass - butt
ass-hat - idiot
ass-pirate - homosexual
assbag - idiot
assbandit - homosexual
assbanger - homosexual
assbite - idiot
assclown - butt
asscock - idiot
asscracker - butt
asses - butts
assface - butt
assfuck - rear-loving
assfucker - homosexual
assgoblin - homosexual
asshat - butt
asshead - idiot
asshole - jerk
asshopper - homosexual
assjacker - homosexual
asslick - idiot
asslicker - Buttlicker
assmonkey - idiot
assmunch - idiot
assmuncher - butt
assnigger - Racial Slur
asspirate - homosexual
assshit - idiot
assshole - butt
asssucker - idiot
asswad - butt
asswipe - butt
bampot - idiot
bastard - illegitimate child
beaner - Mexican
bitch - female dog
bitchass - idiot
bitches - female dogs
bitchtits - homosexual
bitchy - mean
blow job - sexual act
blowjob - sexual act
bollocks - male genitalia
bollox - male genitalia
boner - erection
brotherfucker - homosexual
bullshit - poop
bumblefuck - homosexual
butt plug - cork
butt-pirate - homosexual
buttfucka - homosexual
buttfucker - homosexual
camel toe - female genitalia
carpetmuncher - homosexual
chinc - Chinese
chink - asian
choad - male genitalia
chode - small penis
clit - female genitals
clitface - idiot
clitfuck - sexual act
clusterfuck - mess up
cock - penis
cockass - Jerk
cockbite - idiot
cockburger - idiot
cockface - idiot
cockfucker - idiot
cockhead - idiot
cockjockey - homosexual
cockknoker - homosexual
cockmaster - homosexual
cockmongler - homosexual
cockmongruel - homosexual
cockmonkey - idiot
cockmuncher - homosexual
cocknose - idiot
cocknugget - idiot
cockshit - idiot
cocksmith - homosexual
cocksmoker - homosexual
cocksucker - homosexual
coochie - female genitalia
coochy - female genitalia
coon - African American
cooter - vagina
cracker - Caucasian
cum - semen
cumbubble - idiot
cumdumpster - prostitute
cumguzzler - homosexual
cumjockey - homosexual
cumslut - dirty girl
cumtart - idiot
cunnie - female genitalia
cunnilingus - sexual act
cunt - vagina
cuntface - idiot
cunthole - female genitalia
cuntlicker - homosexual
cuntrag - idiot
cuntslut - idiot
dago - Italian
damn - darn
deggo - Italian
dick - penis
dickbag - idiot
dickbeaters - Hands
dickface - idiot
dickfuck - idiot
dickfucker - homosexual
dickhead - phallace face
dickhole - male genitalia
dickjuice - semen
dickmilk - sperm
dickmonger - homosexual
dicks - penises
dickslap - sexual act
dicksucker - homosexual
dickwad - idiot
dickweasel - idiot
dickweed - idiot
dickwod - idiot
dike - homosexual
dildo - sexual toy
dipshit - idiot
doochbag - idiot
dookie - poop
douche - female hygene product
douche-fag - idiot
douchebag - female hygene accessory
douchewaffle - homosexual
dumass - idiot
dumb ass - idiot
dumbass - idiot
dumbfuck - idiot
dumbshit - idiot
dumshit - idiot
dyke - homosexual
fag - homosexual
fagbag - homosexual
fagfucker - homosexual
faggit - homosexual
faggot - homosexual
faggotcock - homosexual
fagtard - homosexual idiot
fatass - a fat person
fellatio - sexual act
feltch - sexual act
flamer - homosexual
fuck - fornicate
fuckass - idiot
fuckbag - idiot
fuckboy - idiot
fuckbrain - idiot
fuckbutt - butt
fucked - had intercourse
fucker - fornicator
fuckersucker - idiot
fuckface - idiot
fuckhead - butt
fuckhole - jerk
fuckin - sexual act
fucking - freaking
fucknut - idiot
fucknutt - idiot
fuckoff - go away
fucks - sexual act
fuckstick - male genitalia
fucktard - Moron
fuckup - idiot
fuckwad - idiot
handjob - sexual act
hard on - erection
heeb - Jewish Person
hell - heck
ho - woman
hoe - Woman
homo - homosexual
homodumbshit - idiot
honkey - white person
humping - sexual act
fuckwit - dummy
fuckwitt - idiot
fudgepacker - homosexual
gay - homosexual
gayass - butt
gaybob - homosexual
gaydo - homosexual
gayfuck - homosexual
gayfuckist - homosexual
gaylord - homosexual
gaytard - homosexual
gaywad - homosexual
goddamn - goshdarn
goddamnit - goshdarnit
gooch - female genitalia
gook - Chinese
jackass - idiot
jap - japanesse person
jerk off - masturbate
jigaboo - African American
jizz - Semen
jungle bunny - african american
junglebunny - african american
kike - Jewish Person
kooch - female genitalia
kootch - female genitalia
kunt - female genitalia
kyke - Jewish person
lesbian - homosexual
lesbo - homosexual
lezzie - homosexual
mcfagget - homosexual
mick - irish
minge - female genitalia
mothafucka - Jerk
motherfucker - mother lover
motherfucking - fornicating with mother
muff - female genitalia
muffdiver - homosexual
negro - african american
nigga - african american
nigger - african american
niggers - African Americans
niglet - african american child
nut sack - male genitalia
nutsack - male genitalia
paki - pakistanien
panooch - femail genitalia
pecker - Penis
peckerhead - idiot
penis - male genitalia
penisfucker - homosexual
penispuffer - homosexual
piss - urinate
pissed - urinated
pissed off - angry
pissflaps - female genitalia
polesmoker - homosexual
pollock - polish person
poon - female genitals
poonani - female genitalia
poonany - vagina
poontang - female genitalia
porch monkey - african american
porchmonkey - African American
punanny - female genitalia
punta - female dog
pussies - Female Genitalias
pussy - female reproductive organ
pussylicking - sexual act
puto - idiot
queef - vaginal fart.
queer - homosexual
queerbait - homosexual
queerhole - homosexual
rimjob - dirty sexual act
sand nigger - middle eastern
sandnigger - middle eastern
schlong - male genitalia
scrote - male genitalia
shit - poop
shitass - idiot
shitbag - idiot
shitbagger - idiot
shitbrains - idiot
shitbreath - Bad Breath
shitcunt - idiot
shitdick - idiot
shitface - pooface
shitfaced - Drunk
shithead - jerk
shithole - idiot
shithouse - bathroom
shitspitter - butt
shitstain - poop
shitter - defecator
shittiest - worst
shitting - pooping
shitty - bad
shiz - poop
shiznit - poop
skank - dirty girl
skeet - semen
skullfuck - sexual act
slut - sexually popular woman
slutbag - sexually popular woman
smeg - poop
snatch - female genitalia
spic - mexican
spick - mexican american
splooge - ejaculate
tard - mentally challenged
testicle - male genitalia
thundercunt - idiot
tit - breast
titfuck - sexual act
tits - breasts
tittyfuck - sexual act
twat - female genitals
twatlips - idiot
twats - vaginas
twatwaffle - homosexual
unclefucker - homosexual
va-j-j - female genitalia
vag - femail genitalia
vagina - female genitalia
vjayjay - female genitalia
wank - sexual act
wetback - Mexican
whore - hussy
whorebag - idiot
whoreface - idiot
wop - Italian
"""


import re
from collections import defaultdict
vulgarities = []
vulgarities_by_category = defaultdict(list)
for w, c in filter(None, re.findall('^(.*?)\s*-\s*(.*?)\s*\n', X, re.MULTILINE)):
    vulgarities_by_category[c.lower()].append(w)
    vulgarities.append(w)



