pronouns = """
he
she
it
him
her
his
hers
its
they
them
their
theirs
i
me
mine
we
us
our
ours
you
your
yours
one
one's
ones
anyone
anybody
anything
someone
somebody
something
everyone
everybody
everything
nothing
nobody
any
each
either
neither
all
most
some
several
none
both
few
many
himself
herself
itself
themselves
themself
myself
ourselves
ourself
oneself
this
that
these
those
who
whom
which
whose
whoever
whomever
whatever
whichever
what
"""
