# dist.male.first_USCensus1990.lower
male_names = """
James
John
Robert
Michael
William
David
Richard
Charles
Joseph
Thomas
Christopher
Daniel
Paul
Mark
Donald
George
Kenneth
Steven
Edward
Brian
Ronald
Anthony
Kevin
Jason
Matthew
Gary
Timothy
Jose
Larry
Jeffrey
Frank
Scott
Eric
Stephen
Andrew
Raymond
Gregory
Joshua
Jerry
Dennis
Walter
Patrick
Peter
Harold
Douglas
Henry
Carl
Arthur
Ryan
Roger
Joe
Juan
Jack
Albert
Jonathan
Justin
Terry
Gerald
Keith
Samuel
Willie
Ralph
Lawrence
Nicholas
Roy
Benjamin
Bruce
Brandon
Adam
Harry
Fred
Wayne
Billy
Steve
Louis
Jeremy
Aaron
Randy
Howard
Eugene
Carlos
Russell
Bobby
Victor
Martin
Ernest
Phillip
Todd
Jesse
Craig
Alan
Shawn
Clarence
Sean
Philip
Chris
Johnny
Earl
Jimmy
Antonio
Danny
Bryan
Tony
Luis
Mike
Stanley
Leonard
Nathan
Dale
Manuel
Rodney
Curtis
Norman
Allen
Marvin
Vincent
Glenn
Jeffery
Travis
Jeff
Chad
Jacob
Lee
Melvin
Alfred
Kyle
Francis
Bradley
Jesus
Herbert
Frederick
Ray
Joel
Edwin
Don
Eddie
Ricky
Troy
Randall
Barry
Alexander
Bernard
Mario
Leroy
Francisco
Marcus
Micheal
Theodore
Clifford
Miguel
Oscar
Jay
Jim
Tom
Calvin
Alex
Jon
Ronnie
Bill
Lloyd
Tommy
Leon
Derek
Warren
Darrell
Jerome
Floyd
Leo
Alvin
Tim
Wesley
Gordon
Dean
Greg
Jorge
Dustin
Pedro
Derrick
Dan
Lewis
Zachary
Corey
Herman
Maurice
Vernon
Roberto
Clyde
Glen
Hector
Shane
Ricardo
Sam
Rick
Lester
Brent
Ramon
Charlie
Tyler
Gilbert
Gene
Marc
Reginald
Ruben
Brett
Angel
Nathaniel
Rafael
Leslie
Edgar
Milton
Raul
Ben
Chester
Cecil
Duane
Franklin
Andre
Elmer
Brad
Gabriel
Ron
Mitchell
Roland
Arnold
Harvey
Jared
Adrian
Karl
Cory
Claude
Erik
Darryl
Jamie
Neil
Jessie
Christian
Javier
Fernando
Clinton
Ted
Mathew
Tyrone
Darren
Lonnie
Lance
Cody
Julio
Kelly
Kurt
Allan
Nelson
Guy
Clayton
Hugh
Max
Dwayne
Dwight
Armando
Felix
Jimmie
Everett
Jordan
Ian
Wallace
Ken
Bob
Jaime
Casey
Alfredo
Alberto
Dave
Ivan
Johnnie
Sidney
Byron
Julian
Isaac
Morris
Clifton
Willard
Daryl
Ross
Virgil
Andy
Marshall
Salvador
Perry
Kirk
Sergio
Marion
Tracy
Seth
Kent
Terrance
Rene
Eduardo
Terrence
Enrique
Freddie
Wade
Austin
Stuart
Fredrick
Arturo
Alejandro
Jackie
Joey
Nick
Luther
Wendell
Jeremiah
Evan
Julius
Dana
Donnie
Otis
Shannon
Trevor
Oliver
Luke
Homer
Gerard
Doug
Kenny
Hubert
Angelo
Shaun
Lyle
Matt
Lynn
Alfonso
Orlando
Rex
Carlton
Ernesto
Cameron
Neal
Pablo
Lorenzo
Omar
Wilbur
Blake
Grant
Horace
Roderick
Kerry
Abraham
Willis
Rickey
Jean
Ira
Andres
Cesar
Johnathan
Malcolm
Rudolph
Damon
Kelvin
Rudy
Preston
Alton
Archie
Marco
Wm
Pete
Randolph
Garry
Geoffrey
Jonathon
Felipe
Bennie
Gerardo
Ed
Dominic
Robin
Loren
Delbert
Colin
Guillermo
Earnest
Lucas
Benny
Noel
Spencer
Rodolfo
Myron
Edmund
Garrett
Salvatore
Cedric
Lowell
Gregg
Sherman
Wilson
Devin
Sylvester
Kim
Roosevelt
Israel
Jermaine
Forrest
Wilbert
Leland
Simon
Guadalupe
Clark
Irving
Carroll
Bryant
Owen
Rufus
Woodrow
Sammy
Kristopher
Mack
Levi
Marcos
Gustavo
Jake
Lionel
Marty
Taylor
Ellis
Dallas
Gilberto
Clint
Nicolas
Laurence
Ismael
Orville
Drew
Jody
Ervin
Dewey
Al
Wilfred
Josh
Hugo
Ignacio
Caleb
Tomas
Sheldon
Erick
Frankie
Stewart
Doyle
Darrel
Rogelio
Terence
Santiago
Alonzo
Elias
Bert
Elbert
Ramiro
Conrad
Pat
Noah
Grady
Phil
Cornelius
Lamar
Rolando
Clay
Percy
Dexter
Bradford
Merle
Darin
Amos
Terrell
Moses
Irvin
Saul
Roman
Darnell
Randal
Tommie
Timmy
Darrin
Winston
Brendan
Toby
Van
Abel
Dominick
Boyd
Courtney
Jan
Emilio
Elijah
Cary
Domingo
Santos
Aubrey
Emmett
Marlon
Emanuel
Jerald
Edmond
Emil
Dewayne
Will
Otto
Teddy
Reynaldo
Bret
Morgan
Jess
Trent
Humberto
Emmanuel
Stephan
Louie
Vicente
Lamont
Stacy
Garland
Miles
Micah
Efrain
Billie
Logan
Heath
Rodger
Harley
Demetrius
Ethan
Eldon
Rocky
Pierre
Junior
Freddy
Eli
Bryce
Antoine
Robbie
Kendall
Royce
Sterling
Mickey
Chase
Grover
Elton
Cleveland
Dylan
Chuck
Damian
Reuben
Stan
#August
Leonardo
Jasper
Russel
Erwin
Benito
Hans
Monte
Blaine
Ernie
Curt
Quentin
Agustin
Murray
Jamal
Devon
Adolfo
Harrison
Tyson
Burton
Brady
Elliott
Wilfredo
Bart
Jarrod
Vance
Denis
Damien
Joaquin
Harlan
Desmond
Elliot
Darwin
Ashley
Gregorio
Buddy
Xavier
Kermit
Roscoe
Esteban
Anton
Solomon
Scotty
Norbert
Elvin
Williams
Nolan
Carey
Rod
Quinton
Hal
#Brain
Rob
Elwood
Kendrick
Darius
Moises
Son
Marlin
Fidel
Thaddeus
Cliff
Marcel
Ali
Jackson
Raphael
Bryon
Armand
Alvaro
Jeffry
Dane
Joesph
Thurman
Ned
Sammie
Rusty
Michel
Monty
Rory
Fabian
Reggie
Mason
Graham
Kris
Isaiah
Vaughn
Gus
Avery
Loyd
Diego
Alexis
Adolph
Norris
Millard
Rocco
Gonzalo
Derick
Rodrigo
Gerry
Stacey
Carmen
Wiley
Rigoberto
Alphonso
Ty
Shelby
Rickie
Noe
Vern
Bobbie
Reed
Jefferson
Elvis
Bernardo
Mauricio
Hiram
Donovan
Basil
Riley
Ollie
Nickolas
Maynard
Scot
Vince
Quincy
Eddy
Sebastian
Federico
Ulysses
Heriberto
Donnell
Cole
Denny
Davis
Gavin
Emery
Ward
Romeo
Jayson
Dion
Dante
Clement
Coy
Odell
Maxwell
Jarvis
Bruno
Issac
Mary
Dudley
Brock
Sanford
Colby
Carmelo
Barney
Nestor
Hollis
Stefan
Donny
Art
Linwood
Beau
Weldon
Galen
Isidro
Truman
Delmar
Johnathon
Silas
Frederic
Dick
Kirby
Irwin
Cruz
Merlin
Merrill
Charley
Marcelino
Lane
Harris
Cleo
Carlo
Trenton
Kurtis
Hunter
Aurelio
Winfred
Vito
Collin
Denver
Carter
Leonel
Emory
Pasquale
Mohammad
Mariano
Danial
Blair
Landon
Dirk
Branden
Adan
#Numbers
Clair
Buford
German
Bernie
Wilmer
Joan
Emerson
Zachery
Fletcher
Jacques
Errol
Dalton
Monroe
Josue
Dominique
Edwardo
Booker
Wilford
Sonny
Shelton
Carson
Theron
Raymundo
Daren
Tristan
Houston
Robby
Lincoln
Jame
Genaro
Gale
Bennett
Octavio
Cornell
Laverne
Hung
Arron
Antony
Herschel
Alva
Giovanni
Garth
Cyrus
Cyril
Ronny
Stevie
Lon
Freeman
Erin
Duncan
Kennith
Carmine
Augustine
#Young
Erich
Chadwick
Wilburn
Russ
Reid
Myles
Anderson
Morton
Jonas
Forest
Mitchel
Mervin
Zane
Rich
Jamel
Lazaro
Alphonse
Randell
Major
Johnie
Jarrett
Brooks
Ariel
Abdul
Dusty
Luciano
Lindsey
Tracey
Seymour
Scottie
Eugenio
Mohammed
Sandy
Valentin
Chance
Arnulfo
Lucien
Ferdinand
Thad
Ezra
Sydney
Aldo
Rubin
Royal
Mitch
Earle
Abe
Wyatt
Marquis
Lanny
Kareem
Jamar
Boris
Isiah
Emile
Elmo
Aron
Leopoldo
Everette
Josef
Gail
Eloy
Dorian
Rodrick
Reinaldo
Lucio
Jerrod
Weston
Hershel
Barton
Parker
Lemuel
Lavern
Burt
Jules
Gil
Eliseo
Ahmad
Nigel
Efren
Antwan
Alden
Margarito
Coleman
Refugio
Dino
Osvaldo
Les
Deandre
Normand
Kieth
Ivory
Andrea
Trey
Norberto
Napoleon
Jerold
Fritz
Rosendo
Milford
Sang
Deon
Christoper
Alfonzo
Lyman
Josiah
Brant
Wilton
Rico
Jamaal
Dewitt
Carol
Brenton
Yong
Olin
Foster
Faustino
Claudio
Judson
Gino
Edgardo
Berry
Alec
Tanner
Jarred
Donn
Trinidad
Tad
Shirley
Prince
Porfirio
Odis
Maria
Lenard
Chauncey
Chang
Tod
Mel
Marcelo
Kory
Augustus
Keven
Hilario
Bud
Sal
Rosario
Orval
Mauro
Dannie
Zachariah
Olen
Anibal
Milo
Jed
Frances
Thanh
Dillon
Amado
Newton
Connie
Lenny
Tory
Richie
Lupe
Horacio
Brice
Mohamed
Delmer
Dario
Reyes
Dee
Mac
Jonah
Jerrold
Robt
Hank
Sung
Rupert
Rolland
Kenton
Damion
#Chi
Antone
Waldo
Fredric
Bradly
Quinn
Kip
Burl
Walker
Tyree
Jefferey
Ahmed
Willy
Stanford
Oren
Noble
Moshe
Mikel
Enoch
Brendon
Quintin
Jamison
Florencio
Darrick
Tobias
Minh
Hassan
Giuseppe
Demarcus
Cletus
Tyrell
Lyndon
Keenan
Werner
Theo
Geraldo
Lou
Columbus
Chet
Bertram
Markus
Huey
Hilton
Dwain
Donte
Tyron
Omer
Isaias
Hipolito
Fermin
Chung
Adalberto
Valentine
Jamey
Bo
Barrett
Whitney
Teodoro
Mckinley
Maximo
Garfield
Sol
Raleigh
Lawerence
Abram
Rashad
#King
Emmitt
Daron
Chong
Samual
Paris
Otha
Miquel
Lacy
Eusebio
Dong
Domenic
Darron
Buster
Antonia
Wilber
Renato
#Jc
Hoyt
Haywood
Ezekiel
Chas
Florentino
Elroy
Clemente
Arden
Neville
Kelley
Edison
Deshawn
Carrol
Shayne
Nathanial
Jordon
Danilo
Claud
#Val
Sherwood
Raymon
Rayford
Cristobal
Ambrose
Titus
Hyman
Felton
Ezequiel
Erasmo
Stanton
Lonny
#Len
Ike
Milan
Lino
Jarod
Herb
Andreas
Walton
Rhett
Palmer
Jude
Douglass
Cordell
Oswaldo
Ellsworth
Virgilio
Toney
Nathanael
#Del
Britt
Benedict
Mose
Hong
Leigh
Johnson
Isreal
Gayle
Garret
Fausto
#Asa
Arlen
Zack
Warner
Modesto
Francesco
Manual
Jae
Gaylord
Gaston
Filiberto
Deangelo
Michale
Granville
Wes
Malik
Zackary
Tuan
Nicky
Eldridge
Cristopher
Cortez
Antione
Malcom
#Long
Korey
Jospeh
Colton
Waylon
Von
Hosea
Shad
Santo
Rudolf
Rolf
Rey
Renaldo
Marcellus
Lucius
Lesley
Kristofer
Boyce
Benton
#Man
Kasey
Jewell
Hayden
Harland
Arnoldo
Rueben
Leandro
Kraig
Jerrell
Jeromy
Hobert
Cedrick
Arlie
Winford
Wally
Patricia
Luigi
Keneth
Jacinto
Graig
Franklyn
Edmundo
Sid
Porter
Leif
Lauren
Jeramy
Elisha
Buck
Willian
Vincenzo
Shon
Michal
Lynwood
Lindsay
Jewel
Jere
Hai
Elden
Dorsey
Darell
Broderick
Alonso
"""


