prepositions = """
aboard
about
above
absent
across
after
against
along
alongside
amid
amidst
among
amongst
around
as
astride
at
atop
barring
before
behind
below
beneath
beside
besides
between
beyond
but
by
despite
down
during
except
failing
following
for
from
in
inside
into
like
mid
minus
near
next
notwithstanding
of
off
on
onto
opposite
outside
over
past
plus
regarding
round
save
since
than
through
throughout
till
times
to
toward
towards
under
underneath
unlike
until
up
upon
via
with
within
without
according to
ahead of
as to
aside from
because of
close to
due to
far from
in to
inside of
instead of
near to
next to
on to
out of
outside of
owing to
prior to
subsequent to
as far as
as well as
by means of
in accordance with
in addition to
in front of
in place of
in spite of
on account of
on behalf of
on top of
with regard to
in case of
"""
