conjunctions = """
for
and
nor
but
or
yet
so
after
because
although
if
before
since
though
unless
when
now that
even though
only if
while
as
whereas
whether or not
since
in order that
while
even if
until
so
in case
in case that
"""
