stopwords = """
a
aboard
about
above
across
after
against
all
along
alongside
although
amid
amidst
among
amongst
an
and
another
anti
any
anybody
anyone
anything
around
as
astride
at
aught
bar
barring
because
before
behind
below
beneath
beside
besides
between
beyond
both
but
by
circa
concerning
considering
despite
down
during
each
either
enough
everybody
everyone
except
excepting
excluding
few
fewer
following
for
from
he
her
hers
herself
him
himself
his
hisself
i
idem
if
ilk
in
including
inside
into
it
its
itself
like
many
me
mine
minus
more
most
myself
naught
near
neither
nobody
none
nor
nothing
notwithstanding
of
off
on
oneself
onto
opposite
or
other
otherwise
our
ourself
ourselves
outside
over
own
past
pending
per
plus
regarding
round
save
self
several
she
since
so
some
somebody
someone
something
somewhat
such
suchlike
sundry
than
that
the
thee
theirs
them
themselves
there
they
thine
this
thou
though
through
throughout
thyself
till
to
tother
toward
towards
twain
under
underneath
unless
unlike
until
up
upon
us
various
versus
via
vis-a-vis
we
what
whatall
whatever
whatsoever
when
whereas
wherewith
wherewithal
which
whichever
whichsoever
while
who
whoever
whom
whomever
whomso
whomsoever
whose
whosoever
with
within
without
worth
ye
yet
yon
yonder
you
you-all
yours
yourself
yourselves
"""
