
requires_number_arg = [
    ('APARTMENT', 'APT'),
    ('BUILDING', 'BLDG'),
    ('DEPARTMENT', 'DEPT'),
    ('FLOOR', 'FL'),
    ('HANGAR', 'HNGR'),
    ('LOT', 'LOT'),
    ('PIER', 'PIER'),
    ('ROOM', 'RM'),
    ('SLIP', 'SLIP'),
    ('SPACE', 'SPC'),
    ('STOP', 'STOP'),
    ('SUITE', 'STE'),
    ('TRAILER', 'TRLR'),
    ('UNIT', 'UNIT'),
]

# Does not require secondary range number to follow
do_not_require_number_arg = [
    ('BASEMENT', 'BSMT'),
    ('FRONT', 'FRNT'),
    ('LOBBY', 'LBBY'),
    ('LOWER', 'LOWR'),
    ('OFFICE', 'OFC'),
    ('PENTHOUSE', 'PH'),
    ('REAR', 'REAR'),
    ('SIDE', 'SIDE'),
    ('UPPER', 'UPPR'),
]

